'use strict';

const defaultHost = 'localhost';
const defaultPortStart = 9226;
const defaultPortCount = 5;

const monitorInterval = 1000; // ms
var monitorRunning;
var accessibleTargets = {};

function storeOptions (options, callback) {
  const settings = Object.assign({
    host: defaultHost,
    portStart: defaultPortStart,
    portCount: defaultPortCount,
    portChecked: 0,
  }, options);

  chrome.storage.sync.set(settings, callback);
}

function loadOptions (callback) {
  chrome.storage.sync.get({
    host: defaultHost,
    portStart: defaultPortStart,
    portCount: defaultPortCount,
    portChecked: 0,
  }, callback);
}

function parseJson (response) {
  if (response.status !== 200) {
    throw new Error(`Invalid configuration data at ${response.url}`);
  }

  return response.json();
}


function openInspectorTab (data, options) {
  return new Promise((resolve, reject) => {
    // The replace is for older versions. For newer versions, it is a no-op.
    const devtoolsFrontendUrl = data[0].devtoolsFrontendUrl.replace(
      /^https:\/\/chrome-devtools-frontend\.appspot\.com/i,
      'chrome-devtools://devtools/remote'
    );

    const url = new URL(devtoolsFrontendUrl);
    const wsUrl = new URL(data[0].webSocketDebuggerUrl);

    // Update the WebSocket URL with the host and port options. Then, update
    // the DevTools URL with the new WebSocket URL. Also strip the protocol.
    wsUrl.hostname = options.host;
    wsUrl.port = options.portChecked;
    url.searchParams.set('ws', wsUrl.toString().replace('ws://', ''));

    chrome.tabs.create({
      // Without decoding 'ws', DevTools won't load the source files properly.
      url: decodeURIComponent(url.toString())
    }, function onTabCreated(tab) {
      resolve(tab);
    });
  });
}


// Promise an openInspectorTab or reject if the devtools is not open.
function launchDevTools (options) {
  return fetchJson(options.host, options.portChecked)
    .then(function onJson (data) {
        // Server is active, do we need to openInspector?
        return openInspectorTab(data, options);
    }).catch(err => {
      reject(err);
    });
}

function moniterPorts(options) {
  clearPortTargets();

  if (monitorRunning) {
    clearInterval(monitorRunning);
  }
  monitorRunning = setInterval(() => {
    for(var idx = 0; idx < options.portCount; idx++) {
      const targetIdx = idx;
      const port = options.portStart + idx;
      fetchJson(options.host, port)
        .then(data => {
          if(accessibleTargets[targetIdx] != data[0].title) { // 更新title
              accessibleTargets[targetIdx] = undefined;
              updatePortTargets(options);
          }
          accessibleTargets[targetIdx] = data[0].title;
          updatePortTargets(options);
        }).catch(err => {
          accessibleTargets[targetIdx] = undefined;
          updatePortTargets(options);
        });
    }
  }, monitorInterval);
}

function clearPortTargets(options) {
  accessibleTargets = {};

  const htmlPortSet = document.getElementById('portSet')
  htmlPortSet.innerHTML = "No Accessible Target";
}

function updatePortTargets(options) {
  const htmlPortSet = document.getElementById('portSet')

  var needUpdate = false;
  for(var idx = 0; idx < options.portCount; idx++) {
    const port = options.portStart + idx;
    if(accessibleTargets[idx] == undefined
    && htmlPortSet.innerHTML.indexOf(port) == -1) { // 如果不需要显示并且也当前没有显示，则继续判断下一个
      continue;
    } else if(accessibleTargets[idx] != undefined
    && htmlPortSet.innerHTML.indexOf(port) != -1) { // 如果需要显示并且当前已显示，则继续判断下一个
      continue;
    }

    // 需要显示但当前没有显示
    // 或不需要显示但当前已显示（端口删除）
    needUpdate = true;
    break;
  }
  if(needUpdate == false) {
    return;
  }

  var htmlItem = "";
  for(var idx = 0; idx < options.portCount; idx++) {
    if(accessibleTargets[idx] == undefined) {
      continue;
    }

    const port = options.portStart + idx;
    var title = accessibleTargets[idx];
    if (title.length > 20) {
      title = title.substr(0, 10) + '...' + title.substr(title.length-7, title.length);
    }
    if(htmlItem.length != 0){
      htmlItem += '<br/>';
    }
    htmlItem += '<div class="flexRow">';
    htmlItem += '<input name="portRadio" class="portRadio" id="port' + port + '" type="radio" />'
    htmlItem += '<div class="onelinetext">' + port + ' [' +  title + ']</div>'
    htmlItem += '</div>'
  }
  htmlPortSet.innerHTML = htmlItem;

  setDefaultCheck(options.portChecked);
}

function getCheckedPort() {
    var portGroup = document.getElementsByClassName("portRadio");
    for (var i = 0; i < portGroup.length; i++) {
        if (portGroup[i].checked) {
			    return portGroup[i].id.replace('port','');
        };
    }

    return "0";
}

function setDefaultCheck(port) {
  const id = "port" + port;
  const htmlPortChecked = document.getElementById(id);
  if(htmlPortChecked) {
    htmlPortChecked.checked = true;
  }
};


function fetchJson(host, port) {
  const jsonUrl = `http://${host}:${port}/json/list`;

  const promise = new Promise(function(resolve, reject) {
    fetch(jsonUrl)
      .then(response => response.text())
      .then(data => {
        var jsonData = parseJson(data);
        resolve(jsonData);
      }).catch(err => {
        reject(err);
      });
  });

  return promise;
}

function parseJson(data) {
    var jsonData;
    try {
        jsonData = JSON.parse(data);
    } catch(err) {
        if(false === err instanceof SyntaxError) {
            reject(err);
        }
        data = data.replace(/"title":.*?",/, '"title":"JsView",');
        jsonData = JSON.parse(data);
    };

    return jsonData;
}
